require 'spec_helper'

describe 'Hawkei' do
  let(:processor_mock) { double('Processor', enqueue: true) }

  describe '.configurations' do
    it { expect(Hawkei.configurations).to be_a(Hawkei::Config) }
  end

  describe '.track' do
    let(:expected_arg) {{
      user_id: 42,
      name:    'test',
      message_id: kind_of(String),
      timestamp: kind_of(String),
      session_tracker_id: kind_of(String),
      properties: {
        total: 10
      },
      library: {
        name: "hawkei",
        language: "ruby",
        version: kind_of(String)
      },
      server: {
        host: kind_of(String),
        pid: kind_of(Numeric),
        software: 'rspec',
      },
    }}

    context 'async' do
      before do
        allow(Hawkei).to receive(:processor).and_return(processor_mock)
        Hawkei.track('test', { user_id: 42, properties: { total: 10 }}, no_ssl: true, space_name: 'test')
      end

      it { expect(processor_mock).to have_received(:enqueue).with(action: 'track_events:create', payload: hash_including(expected_arg), options: { space_name: 'test' }) }
    end

    context 'not enabled' do
      before do
        Hawkei.configurations.enabled = false
        Hawkei.track('test', { user_id: 42, properties: { total: 10 }}, no_ssl: true, space_name: 'test')
      end

      it { expect(processor_mock).to_not have_received(:enqueue) }
    end
  end

  describe '.identify' do
    let(:expected_arg) {{
      tracker_id: 'xx',
      properties: {
        name: 'John Doe'
      },
      user_id: 42,
      message_id: kind_of(String),
      timestamp: kind_of(String),
      session_tracker_id: kind_of(String),
      library: {
        name: "hawkei",
        language: "ruby",
        version: kind_of(String)
      },
      server: {
        host: kind_of(String),
        pid: kind_of(Numeric),
        software: 'rspec',
      },
    }}

    context 'async' do
      before do
        allow(Hawkei).to receive(:processor).and_return(processor_mock)
        Hawkei.identify(42, { tracker_id: 'xx', properties: { name: 'John Doe' }}, no_ssl: true, space_name: 'test')
      end

      it { expect(processor_mock).to have_received(:enqueue).with(action: 'identify_events:create', payload: hash_including(expected_arg), options: { space_name: 'test' }) }
    end

    context 'not enabled' do
      before do
        Hawkei.configurations.enabled = false
        Hawkei.identify(42, { tracker_id: 'xx', properties: { name: 'John Doe' }}, no_ssl: true, space_name: 'test')
      end

      it { expect(processor_mock).to_not have_received(:enqueue) }
    end
  end

  describe '.group' do
    let(:expected_arg) {{
      properties: {
        name: 'John Group'
      },
      message_id: kind_of(String),
      timestamp: kind_of(String),
      session_tracker_id: kind_of(String),
    }}

    context 'async' do
      before do
        allow(Hawkei).to receive(:processor).and_return(processor_mock)
        Hawkei.group(42, { properties: { name: 'John Group' }}, no_ssl: true, space_name: 'test')
      end

      it { expect(processor_mock).to have_received(:enqueue).with(action: 'group_events:create', payload: hash_including(expected_arg.merge(group_id: 42)), options: { space_name: 'test' }) }
    end

    context 'not enabled' do
      before do
        Hawkei.configurations.enabled = false
        Hawkei.group(42, { properties: { name: 'John Group' }}, no_ssl: true, space_name: 'test')
      end

      it { expect(processor_mock).to_not have_received(:enqueue) }
    end
  end

  describe '.watch' do
    let(:expected_arg) {{
      template_flow: 'My Hawkei',
      expected_times: 42,
      message_id: kind_of(String),
      timestamp: kind_of(String),
      session_tracker_id: kind_of(String),
    }}
    let(:response) { double('Hawkei::Watcher', successful?: true) }

    context 'async' do
      before do
        allow(Hawkei).to receive(:processor).and_return(processor_mock)
        Hawkei.watch('My Hawkei', { expected_times: 42 }, no_ssl: true, space_name: 'test')
      end

      it { expect(processor_mock).to have_received(:enqueue).with(action: 'watchers:create', payload: hash_including(expected_arg), options: { space_name: 'test' }) }
    end

    context 'without async' do
      before do
        allow(Hawkei::Watcher).to receive(:create).and_return(response)
        Hawkei.watch('My Hawkei', { expected_times: 42 }, no_ssl: true, space_name: 'test', sync: true)
      end

      it { expect(Hawkei::Watcher).to have_received(:create).with(hash_including(expected_arg), { no_ssl: true, space_name: 'test', sync: true }) }
    end

    context 'not enabled' do
      before do
        Hawkei.configurations.enabled = false
        Hawkei.watch('My Hawkei', { expected_times: 42 }, no_ssl: true, space_name: 'test')
      end

      it { expect(Hawkei::Watcher).to_not receive(:create) }
    end

    context 'without session' do
      let(:expected_arg) {{
        template_flow: 'My Hawkei',
        expected_times: 42,
        message_id: kind_of(String),
        timestamp: kind_of(String),
      }}

      before do
        allow(Hawkei).to receive(:processor).and_return(processor_mock)
        Hawkei.watch('My Hawkei', { expected_times: 42, without_session: true }, no_ssl: true, space_name: 'test')
      end

      it { expect(processor_mock).to have_received(:enqueue).with(action: 'watchers:create', payload: hash_including(expected_arg), options: { space_name: 'test' }) }
    end
  end

end
